#include "dot1x.h"

#include "pcap.h"
#include "md5.h"
#include <stdlib.h>

#include <ctime>
#include <netinet/in.h>

#include <sys/types.h>
#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/times.h>
#include <net/if.h>
#include <netinet/in.h>
#include <net/if_arp.h>
#include <arpa/inet.h>
#include <string>
#include <cstring>
#include <iostream>

#define LOWORD(i)((WORD)(i))  
#define HIWORD(i)((WORD)(((DWORD)(i)>>16)&0xffff)) 

#define LOBYTE(i)((BYTE)(i))  
#define HIBYTE(i)((BYTE)(((WORD)(i)>>8)&0xff)) 

long GetTickCount()
{
	tms tm;
	return times(&tm);
}


namespace NetworkOperation 
{
	using namespace std;

	int GetSystemInterfaceInfo(InterfaceInfo * infoarray) 
	{
		int fd, intrface, retn = 0;
		struct ifreq buf[MAXINTERFACES];
		struct arpreq arp;
		struct ifconf ifc;

		int interfacenum;
		if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) >= 0) 
		{
			ifc.ifc_len = sizeof(buf);
			ifc.ifc_buf = (caddr_t) buf;
			if (!ioctl(fd, SIOCGIFCONF, (char *) &ifc)) 
			{
				interfacenum = ifc.ifc_len / sizeof(struct ifreq);

				for (int i = 0; i < interfacenum; i++) 
				{
					infoarray[i].name = buf[i].ifr_name;

					/*Get IP of the net card */

					if (!(ioctl(fd, SIOCGIFADDR, (char *) &buf[i])))
					{
						infoarray[i].ipaddr = inet_ntoa(
								((struct sockaddr_in *) 
								 (&buf[i].ifr_addr))->sin_addr);
					} else 
					{
						infoarray[i].ipaddr = "";
					}

					/*Get HW ADDRESS of the net card */

					if (!(ioctl(fd, SIOCGIFHWADDR, (char *) &buf[i]))) 
					{
						for (int j = 0; j < 6; j++) 
						{
							infoarray[i].macaddr[j] =
								(unsigned char) buf[i].ifr_hwaddr.sa_data[j];
						}

					} else 
					{
						infoarray[i].macaddr[0] = -1;
					}
				}

			}
			else 
			{
				interfacenum = -1;
			}
		}
		else 
		{
			interfacenum = -2;
		}

		close(fd);

		return interfacenum;
	}

	HuaweiNetwork::HuaweiNetwork(const string &username, const string &passwd,
			const string &interfacename, int *ip, int *mac, int bRenew)
	{
		USERDATA *m_pData = &m_Data;
		strcpy(m_pData->username, username.c_str());
		strcpy(m_pData->password, passwd.c_str());
		strcpy(m_pData->nic, interfacename.c_str());

		m_pData->ip[0] = ip[0];
		m_pData->ip[1] = ip[1];
		m_pData->ip[2] = ip[2];
		m_pData->ip[3] = ip[3];

		m_pData->mac[0] = mac[0];
		m_pData->mac[1] = mac[1];
		m_pData->mac[2] = mac[2];
		m_pData->mac[3] = mac[3];
		m_pData->mac[4] = mac[4];
		m_pData->mac[5] = mac[5];

		m_pData->morb = 0x00;
		m_pData->dhcp = bRenew;
		m_pData->multisend = 0x00;
		m_pData->updateip = 0x01;
		m_pData->relogin = 0x00;

		m_fp = NULL;		/* network interface device */
		/* multicasting address */
		u_char mcast[6] = { 0x01, 0x80, 0xc2, 0x00, 0x00, 0x03 };
		/* default address is  multicasting address */
		memcpy(m_DestMac, mcast, 6);
		char cver[14] =
		{ 'C', 'H', ' ', 'V', '2', '.', '4', '0', '-', '0', '3', '3', '5', 0 };
		memcpy(m_ClientVersion, cver, 14);
		//srand(time(0));
	}

	void HuaweiNetwork::run()
	{
		struct pcap_pkthdr *header;
		const u_char *pkt_data;

		int res;
		time_t dwTick, dwOldTick;
		dwTick = dwOldTick = clock();
		int flag = 0;

		/*  read frames sequentially */
		while ((res = pcap_next_ex(m_fp, &header, &pkt_data)) >= 0) 
		{
			if (res == 0) 
			{	// Timeout elapsed
				/* use ::GetTickCount() to get the elapsed time */
				dwTick = clock();
				if (dwTick - dwOldTick >= 90000	//ms
						&& m_Data.relogin == 1) {
					message("Reconnecting...");
					dwOldTick = clock();
				}
				continue;
			}

			PPKTHDR pbuf = (PPKTHDR) pkt_data;
			/* get the source mac address of the frame received */
			memcpy(m_DestMac, pbuf->SourMAC, 6);	

			if (pbuf->Code == EAP_REQUEST) 
			{
				switch (pbuf->EapType) 
				{
					case EAP_AVAILABLE:			/* 保持在线包 */
						message("EAP_AVAILABLE received.");
						if(SendKeeponline(pbuf->Id))
							message("EAP_AVAILABLE sended."); 
						dwOldTick = clock();
						//arak 1+d
						break;
					case EAP_NOTIFICATION:		/* 版本号包 */
						message("EAP_NOTIFICATION received.");
						SendVersion(pbuf->Id);
						break;
					case EAP_IDENTITY:
						message("EAP_IDENTIFY received.");
						if(SendUsername(pbuf->Id))
							message("EAP_IDENTIFY sended.");
						//arak 1+d
						break;
					case EAP_MD5:
						message("EAP_MD5 received.");
						SendPassword(pbuf->Id,
								((PPASSWORDFRM) pkt_data)->Md5Pwd);
						break;
					default:;
				}

				continue;
			}

			if (pbuf->Code == EAP_SUCCESS) {

				message("Successfully Login.");
				//                          isonline = true;
				if(m_Data.dhcp)
					Connected();
				continue;
			}

			if (pbuf->Code == EAP_FAILURE) {

				if (pbuf->EapType == EAP_LOGOUT) 
				{
					message("Logout!");
					//ConnectionInterrupted();
				} 
				else 
				{
					// Network Failure;
					message((const char *) (pkt_data + 0x18));
					// arak 1+d
					//ConnectionInterrupted();
				}
				break;		//jump out of the loop
			}

			if (pbuf->Code == EAP_OTHER) 
			{	//there are three frames to handle

				//获取Token
				/*if(GetToken((PTOKENFRM)pkt_data))
					continue;*/

				//the former is about the client download address, ignore it
				if (flag == 0) 
				{
					flag = 1;
					continue;
				}
				//append information
				u_char a[0xff] = { 0 };
				memcpy(a, pkt_data + 0x1a, *(pkt_data + 0x11) - 4);
				//a[*(pkt_data +0x11)-3] = '\0';
				for (int i = 0; i < 0xff; i++) {
					if (a[0xff - i] == 0x34) {
						a[0xff - i] = '\t';
						a[0xff - i + 1] = '\n';
						break;
					}
				}
				message((const char *)a);

				continue;
			}
		}
		CloseAdapter();
	}

	bool HuaweiNetwork::Connect() {
		message("Initializing Network Adapter...");
		if (!OpenAdapter()) {	/* open adapter, set filter and start the thread */
			message("Network Adapter Initializing Failed.");
			return false;
		}

		//start(QThread::HighPriority);

		SendLogin();

		//          Connected();

	}

	void HuaweiNetwork::DisConnect() {
		//          ConnectionInterrupted();
		if (m_fp) {
			SendLogout();
			message("Disconnected.");
		}
	}

	bool HuaweiNetwork::OpenAdapter() {
		bpf_u_int32 netmask = 0;
		char pcap_filter[100];	//filter space
		struct bpf_program pcap_fp;	//hold the compiled filter.
		char errbuf[PCAP_ERRBUF_SIZE] = "";
		//open adapter
		if (!(m_fp = pcap_open_live(m_Data.nic,	// name of the device
						256,	// portion of the packet to capture, max 65536
						0,	// promiscuous mode closed 我改成了open
						10,	// read timeout
						errbuf)))	// error buffer
		{
			//              AfxmessageBox(errbuf);
			message((const char *) errbuf);
			return false;
		}
		//set filter to receive frame of 802.1X only
		/*sprintf(pcap_filter,
				"ether dst %x:%x:%x:%x:%x:%x and ether proto 0x888e",
				m_Data.mac[0], m_Data.mac[1], m_Data.mac[2], m_Data.mac[3],
				m_Data.mac[4], m_Data.mac[5]);*/
		/* 把过滤器设置为抓本网卡和h3c服务器多播的包，因为服务器没隔30秒
		 * 会发送一个请求用户名的包
		 * */
		sprintf(pcap_filter, 
				"ether dst %x:%x:%x:%x:%x:%x or ether dst 01:80:c2:00:00:03 and ether proto 0x888e",
				m_Data.mac[0],m_Data.mac[1],m_Data.mac[2],
				m_Data.mac[3],m_Data.mac[4],m_Data.mac[5]);

		if (pcap_compile(m_fp, &pcap_fp, pcap_filter, 0, netmask) == -1)
			return false;

		if (pcap_setfilter(m_fp, &pcap_fp) == -1)
			return false;

		return true;
	}

	void HuaweiNetwork::CloseAdapter() 
	{
		if (m_fp != 0) {
			pcap_close(m_fp);	//shutdown the apdater
			m_fp = 0;
		}
	}

	void HuaweiNetwork::SetMd5Buf(PPASSWORDFRM pBuf, const u_char ID, const u_char * chap) {	//digest MD5
		u_char TmpBuf[1 + 64 + 16];
		MD5_CTX md5T;
		u_char digest[16];
		int PasswdLen = strlen(m_Data.password);
		TmpBuf[0] = ID;		//   memcpy(TmpBuf + 0x00, ID, 1);
		memcpy(TmpBuf + 0x01, m_Data.password, PasswdLen);
		memcpy(TmpBuf + 0x01 + PasswdLen, chap, 16);
		md5T.MD5Update(TmpBuf, 17 + PasswdLen);
		md5T.MD5Final(digest);
		memcpy(pBuf->Md5Pwd, digest, 16);
	}

	void HuaweiNetwork::InitBuf(u_char * buf) {	//initial every frame
		u_char prototype[3] = { 0x88, 0x8e, 0x01 };

		if (m_Data.multisend == 0x00)	//set destination MAC
			memcpy(buf, m_DestMac, 6);
		memcpy(buf + 6, m_Data.mac, 6);	//set source MAC
		memcpy(buf + 12, prototype, 3);	//set protocol type and its version
	}

	bool HuaweiNetwork::SendLogin() {	//send the EAPOL-START frame
		u_char buf[100] = { 0 };
		PLOGINFRM pbuf = (PLOGINFRM) buf;

		InitBuf(buf);		//Set Dest MAC, Source MAC, Protocol Type and Version
		if (m_Data.morb == 'b')
			memset(buf, 0xff, 6);	//broadcast EAPOL-START (by default is multicast)
		pbuf->PktType = EAPOL_START;	//EAPOL-START

		if (!pcap_sendpacket(m_fp, buf, 60)) {
			message("Login sent");
			return true;
		}

		return false;
	}

	bool HuaweiNetwork::SendLogout() {	//send logout frame
		u_char buf[100] = { 0 };
		PLOGOUTFRM pbuf = (PLOGOUTFRM) buf;

		InitBuf(buf);		//set Dest MAC, Source MAC, Protocol Type and Version
		pbuf->PktType = EAPOL_LOGOFF;	//EAPOL-LOGOUT

		if (!pcap_sendpacket(m_fp, buf, 60)) {
			message("Disconnecting...");
			return true;
		}

		return false;
	}

	bool HuaweiNetwork::SendVersion(const u_char Id) 
	{	//send frame to confirm version 
		u_char buf[100] = { 0 };
		PVERSIONFRM pbuf = (PVERSIONFRM) buf;
		InitBuf(buf);
		pbuf->Hdr.PktType = EAP;
		pbuf->Hdr.Len1 = htons(sizeof(PKTHDR) - sizeof(EAPHEADER) + 22);
		pbuf->Hdr.Code = EAP_RESPONSE;			/* eap_response 包*/
		pbuf->Hdr.Id = 0x01;
		pbuf->Hdr.Len2 = pbuf->Hdr.Len1;
		pbuf->Hdr.EapType = EAP_NOTIFICATION;
		GenerateVersion(pbuf->Version);
		// \arak +n
		InitBuf(buf);
		if (!pcap_sendpacket(m_fp, buf, sizeof(VERSIONFRM))) 
		{
			message("Finding authentication server...");
			return true;
		}

		return false;
	}
	//    #include <sys/socket.h>
	bool HuaweiNetwork::SendUsername(const u_char Id) 
	{	//send username frame
		u_char buf[100] = { 0 };
		PUSERNAMEFRM pbuf = (PUSERNAMEFRM) buf;

		InitBuf(buf);
		pbuf->Hdr.Len1 = htons(sizeof(USERNAMEFRM) - sizeof(EAPHEADER)
				          -20 + strlen(m_Data.username));
		pbuf->Hdr.Code = EAP_RESPONSE;
		pbuf->Hdr.Id = Id;
		pbuf->Hdr.Len2 = pbuf->Hdr.Len1;
		pbuf->Hdr.EapType = EAP_IDENTITY;
		pbuf->Unknown[0] = 0x15;
		pbuf->Unknown[1] = 0x04;
		if (m_Data.updateip == 0x01)
			memcpy(pbuf->Ip, m_Data.ip, 4);	//upload host IP
		pbuf->Unknown2[0] = 0x06;
		pbuf->Unknown2[1] = 0x07;
		GenerateBase64(pbuf->Base64);
		memcpy(&pbuf->Username, &m_Data.username, strlen(m_Data.username));

		return (!pcap_sendpacket(m_fp, buf, sizeof(USERNAMEFRM) -20 
								+ strlen(m_Data.username))); 
	}

	bool HuaweiNetwork::SendPassword(const u_char Id, const u_char * Chap) 
	{//send passwd frame
		u_char buf[100] = { 0 };
		PPASSWORDFRM pbuf = (PPASSWORDFRM) buf;

		InitBuf(buf);
		pbuf->Hdr.Len1 = htons(strlen(m_Data.username) + 0x16);
		pbuf->Hdr.Code = EAP_RESPONSE;
		pbuf->Hdr.Id = Id;
		pbuf->Hdr.Len2 = pbuf->Hdr.Len1;
		pbuf->Hdr.EapType = EAP_MD5;
		pbuf->Unknown[0] = 0x10;								/* md5 的长度 */
		SetMd5Buf(pbuf, Id, Chap);
		memcpy(pbuf->Username, m_Data.username, strlen(m_Data.username));

		if (!pcap_sendpacket(m_fp, buf, 60)) {
			message("Verifying Password ...");
			return true;
		}

		return false;
	}

	//Written By AGanNo2 (AGanNo2@163.com)
	void HuaweiNetwork::GenerateBase64(u_char *buf)
	{
		srand(time(0));
		unsigned long magic = rand();
		//unsigned long uRand = rand((unsigned long)time(NULL));
		//unsigned long magic = uRand * GetTickCount();
		//unsigned long magic = 0x392df6ea;
		////初始化strMagic,把magic 的16 进制形式转化为字符串存放其中
		char strMagic[9]={0};
		unsigned char strTemp[4] = {0};
		memcpy(strTemp,(unsigned char *)&magic,4);
		sprintf(strMagic,"%02x%02x%02x%02x",strTemp[0],strTemp[1],
				strTemp[2],strTemp[3]);
		//printf("%s\n",strMagic);
		//初始化版本信息
		unsigned char version[20];
		memset(version,0,sizeof(version));
		//memcpy(version,"CH V2.40-0335",strlen("CH V2.40-0335"));
		memcpy(version, m_ClientVersion, strlen(m_ClientVersion));
		memcpy(version+16,(unsigned char *)&magic,4);
		//经过下面两次加密得到那20 个字节的随机数
		EncodeVersion(strMagic,version,0x10);
		EncodeVersion("HuaWei3COM1X",version,0x14);
		//对得到的20 个字节进行base64 编码得到28 个字节的结果
		EncodeBase64(version, (char *)buf, 20);
		//填充最后两个字节
		buf[28] = 0x20;
		buf[29] = 0x20;
	}

	//Written By AGanNo2 (AGanNo2@163.com)
	int HuaweiNetwork::EncodeBase64(const unsigned char* pSrc, 
			char* pDst, int nSrcLen)
	{
		const char EnBase64Tab[] =
			"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
			"abcdefghijklmnopqrstuvwxyz0123456789+/";
		unsigned char c1;
		unsigned char c2;
		unsigned char c3; // 输入缓冲区读出3 个字节
		int nDstLen = 0; // 输出的字符计数
		int nDiv = nSrcLen / 3; // 输入数据长度除以3 得到的倍数
		int nMod = nSrcLen % 3; // 输入数据长度除以3 得到的余数
		// 每次取3 个字节，编码成4 个字符
		for (int i = 0; i < nDiv; i ++)
		{
			// 取3 个字节
			c1 = *pSrc++;
			c2 = *pSrc++;
			c3 = *pSrc++;
			//编码成4 个字符
			*pDst++ = EnBase64Tab[c1 >> 2];
			*pDst++ = EnBase64Tab[((c1 << 4) | (c2 >> 4)) & 0x3f];
			*pDst++ = EnBase64Tab[((c2 << 2) | (c3 >> 6)) & 0x3f];
			*pDst++ = EnBase64Tab[c3 & 0x3f];
			nDstLen += 4;
		}
		// 编码余下的字节
		if (nMod == 1)
		{
			c1 = *pSrc++;
			*pDst++ = EnBase64Tab[(c1 & 0xfc) >> 2];
			*pDst++ = EnBase64Tab[((c1 & 0x03) << 4)];
			*pDst++ = '=';
			*pDst++ = '=';

			//nLineLen += 4;
			nDstLen += 4;
		}
		else if (nMod == 2)
		{
			c1 = *pSrc++;
			c2 = *pSrc++;
			*pDst++ = EnBase64Tab[(c1 & 0xfc) >> 2];
			*pDst++ = EnBase64Tab[((c1 & 0x03) << 4) | ((c2 & 0xf0) >> 4)];
			*pDst++ = EnBase64Tab[((c2 & 0x0f) << 2)];
			*pDst++ = '=';
			nDstLen += 4;
		}
		// 输出加个结束符
		*pDst = '\0';
		return nDstLen;
	}
	void HuaweiNetwork::GenerateVersion(u_char * buf) 
	{
		buf[0] = 0x01;
		buf[1] = 0x16;

		srand(time(0));
		unsigned long magic = rand();
		//initial strMagic, store the of string of the hexadecimal value of magic in it*/
		char strMagic[9] = { 0 };
		unsigned char strTemp[4] = { 0 };
		memcpy(strTemp, (unsigned char *) &magic, 4);
		sprintf(strMagic, "%02x%02x%02x%02x", strTemp[0], strTemp[1],
				strTemp[2], strTemp[3]);
		//printf("%s\n",strMagic);
		//initial info of version
		unsigned char version[20];
		memset(version, 0, sizeof(version));
		memcpy(version, m_ClientVersion, strlen(m_ClientVersion));
		memcpy(version + 16, (unsigned char *) &magic, 4);

		//set the variable 20 bytes of the information about version 
		EncodeVersion(strMagic, version, 0x10);
		EncodeVersion("HuaWei3COM1X", version, 0x14);
		memcpy(buf + 2, version, 20);

		//debug 2
		//for(int i=0; i<20; i++)
		//            {printf("%02x ",(unsigned char)version[i]);}
		/*buf[22] = 0x02;
		  buf[23] = 0x16;

		//the last 20 bytes
		char winVersion[20] = { 
		0x3a, 0x71, 0x38, 0x01, 0x0b, 0x3b, 0x7e, 0x3d,
		0x26, 0x7c, 0x7c, 0x17, 0x0b, 0x46, 0x08, 0x32,
		0x32, 0x08, 0x46, 0x0b };
		//unsigned long uWinVersion = GetVersion();
		//sprintf(winVersion,"%u",uWinVersion);
		//EncodeVersion("HuaWei3COM1X", (unsigned char *)winVersion, 0x14);
		memcpy(buf + 24, winVersion, 20);
		*/
	}

	//Written By AGanNo2 (AGanNo2@163.com)
	// use strConst to encrypt strDest
	void HuaweiNetwork::EncodeVersion(char *strConst, 
			unsigned char *strDest, int iSize) 
	{

		char *temp = new char[iSize];
		int iTimes = iSize / strlen(strConst);
		for (int i = 0; i < iTimes; i++)
			memcpy(temp + i * strlen(strConst), strConst,
					strlen(strConst));

		memcpy(temp + iTimes * strlen(strConst), strConst,
				iSize % strlen(strConst));

		for (int i = 0; i < iSize; i++)
			strDest[i] = strDest[i] ^ temp[i];

		for (int i = 0; i < iSize; i++)
			strDest[iSize - i - 1] = strDest[iSize - i - 1] ^ temp[i];

		delete[]temp;
	}
	/* dhcp 自动获取ip  */
	void HuaweiNetwork::Connected()
	{
		system("/sbin/dhclient");
	}
	void HuaweiNetwork::message(const char *msg)
	{
		cout << msg << endl;
	}

	// deal with the KeeponlineFrm

	bool HuaweiNetwork::SendKeeponline(const u_char Id) 
	{
		u_char buf[100] = { 0 };
		PKEEPONLINEFRM pbuf = (PKEEPONLINEFRM) buf;

		InitBuf(buf);
		//pbuf->Hdr.Len1 = htons(strlen(m_Data.username) + 0x40);
		pbuf->Hdr.PktType = EAP;
		pbuf->Hdr.Len1 = htons(sizeof(KEEPONLINEFRM) - sizeof(EAPHEADER) - 20
									+ strlen(m_Data.username));
		pbuf->Hdr.Code = EAP_RESPONSE;
		pbuf->Hdr.Id = Id;
		pbuf->Hdr.Len2 = pbuf->Hdr.Len1;
		pbuf->Hdr.EapType = EAP_AVAILABLE;

		pbuf->UseProxy = 0x00;
		pbuf->Unknown1[0] = 0x15;
		pbuf->Unknown1[1] = 0x04;
		//memcpy(pbuf->Magic, m_Token, 32);
		if (m_Data.updateip == 0x01)
			memcpy(pbuf->Ip, m_Data.ip, 4);
		pbuf->Unknown2[0] = 0x06;
		pbuf->Unknown2[1] = 0x07;
		GenerateBase64(pbuf->Base64);
		memcpy(pbuf->Username, m_Data.username, strlen(m_Data.username));

		return !(pcap_sendpacket(m_fp, buf, sizeof(KEEPONLINEFRM) - 20
								+ strlen(m_Data.username)));
	}

	/*bool HuaweiNetwork::GetToken(PTOKENFRM buf)
	{
		PTOKENFRM pbuf = (PTOKENFRM)buf;
		//判断是否为所需数据包 0x23 0x44 0x23 0x31
		if(pbuf->Identifier[0] == 0x23 &&
				pbuf->Identifier[1] == 0x44 &&
				pbuf->Identifier[2] == 0x23 &&
				pbuf->Identifier[3] == 0x31)
		{
			memcpy(m_Token,pbuf->Token,33); //将服务器发来的Token串赋给m_Token
			m_Token[33] = 0;

			GenerateFinalMagic(m_Token);
			return true;
		}

		return false;
	}*/

/*	void HuaweiNetwork::GenerateMagic(u_char *buf)
	{	//分四次进行生成Magic
		for(int i=0;i<4;i++)
		{
			CalcASC(buf + 8 * i);
		}
	}

	void HuaweiNetwork::GenerateFinalMagic(u_char *buf)
	{
		GenerateMagic(buf);

		MD5_CTX md5T;
		//Magic的MD5值作为前16个字节
		md5T.MD5Update(m_Token,32);
		md5T.MD5Final(m_Token);
		m_Token[16] = 0;
		//再次MD5加密作为后16个字节
		md5T.MD5Update(m_Token,16);
		md5T.MD5Final(m_Token + 16);
	}

	void HuaweiNetwork::CalcASC(u_char *buf)
	{	//H3C的算法，所得结果为ASCII字符串
		WORD Res;
		DWORD dEBX,dEBP;
		DWORD dEDX = 0x10;
		DWORD mSalt[4] = {0x56657824,0x56745632,0x97809879,0x65767878};

		DWORD dECX = *((DWORD*)buf);
		DWORD dEAX = *((DWORD*)(buf + 4));

		dEDX *= 0x9E3779B9;

		while(dEDX != 0)
		{
			dEBX = dEBP = dECX;
			dEBX >>= 5;
			dEBP <<= 4;
			dEBX ^= dEBP;
			dEBP = dEDX;
			dEBP >>= 0x0B;
			dEBP &= 3;
			dEBX += mSalt[dEBP];
			dEBP = dEDX;
			dEBP ^= dECX;
			dEDX += 0x61C88647;
			dEBX += dEBP;
			dEAX -= dEBX;
			dEBX = dEAX;
			dEBP = dEAX;
			dEBX >>= 5;
			dEBP <<= 4;
			dEBX ^= dEBP;
			dEBP = dEDX;
			dEBP &= 3;
			dEBX += mSalt[dEBP];
			dEBP = dEDX;
			dEBP ^= dEAX;
			dEBX += dEBP;
			dECX -= dEBX;
		}


		Res = LOWORD(dECX);
		*buf = LOBYTE(Res);
		*(buf+1) = HIBYTE(Res);

		Res = HIWORD(dECX);
		*(buf+2) = LOBYTE(Res);
		*(buf+3) = HIBYTE(Res);

		Res = LOWORD(dEAX);
		*(buf+4) = LOBYTE(Res);
		*(buf+5) = HIBYTE(Res);

		Res = HIWORD(dEAX);
		*(buf+6) = LOBYTE(Res);
		*(buf+7) = HIBYTE(Res);
	}*/

	}
